import cPickle
import os
import os.path
import string
import copy
import logging
import sys
import kraken.Constants
import datetime
from kraken.entrytypes.Entry import loadKraEntry
import kraken.tools.TextUi as TextUi
from EntryBase import EntryBase
import MetaDataDescriptor
from TypicalNamespaces import *

class MetaData:

    def __init__(self):
       self.entries = [] # the loaded data
       self.keywords = []
       self.descriptors = MetaDataDescriptor.getMetaDataDescriptors()
    
    def writeToRdf(self, fileName):
        base = EntryBase()
        store = base.createStore()
        for entry in self.entries:
            entry.writeEntryToRdf(store=store, fileName=None)
            # and now write the secondaries
            for key in entry.secondaries.keys():
                en2 = entry.secondaries[key]
                en2.writeEntryToRdf(store=store, fileName=None)
                # and add a transect link
                store.add((entry.getResourceURI(), 
                           NS_KRAKEN["secondary"],
                           en2.getResourceURI()))
        base.saveStoreAndPostProcess(store, fileName)
     
    
    def addEntry(self, entry):
        self.entries.append(copy.deepcopy(entry))
        self.__updateKeywords(entry)

    def getEntryTypes(self):
        return [
           "Article",
           "Book",
           "Person",
           "Webpage",           
           "Online Article",
           "Research Proposal",
           "ResearchIdea",
           "Appointment",
           "To Do Item",
           "Conference",
           "Journal",
           "Research Group",
           "Software",
           "Documentation",
           "Document",
           "Journal Entry",
           "Meeting"           
        ]  
        
        
    def getAllPropertyNames(self):   
        lst = []
        for entry in self.entries:
            a = entry.listExtendedPropertyNames()
            # print a
            for pname in a:
              if not pname in lst:
                  lst.append(pname)
        #print lst
        return lst        
        
        
    '''
      Updates an older occurence of the entry with the new one.
      Used for in-place updating for annotate
    '''
    def updateEntry(self, entry):
        counter = 0
        for a in self.entries:
            if a.getResourceURI() == entry.getResourceURI():
                print "Entry found, updating..."
                self.entries[counter] = entry
                break
            counter += 1
        self.__updateKeywords(entry)
                
        
    def load(self, system, runtime, rewrite = 0):
        for directory in system.getProperty("krakenDirs"):
           print "Processing directory "+ directory
           if not os.path.isdir(directory):
               logging.critical("Directory "+ directory + " does not exist! Skipping...")
               return
           for x in os.listdir(directory):
               self.loadEntryIntoMetadata(directory, x, system, runtime, rewrite)        
        
    def loadEntryIntoMetadata(self, directory, x, system, runtime, rewrite):
           entryDir = os.path.join(directory, x)
           if not os.path.isdir(entryDir):
               return
           # try if the kraken file is there
           entry = loadKraEntry(system, directory, x)
           if entry == None:
               return
           oldDir = entry.getProperty("directoryName")
           newDir = x
           if oldDir != newDir:               
               print "Olddir = " + oldDir + "\nNew dir "+ newDir
               entry.addProperty("directoryName", newDir)
               entry.writeRDF(entryDir)
           # This is for converting old style entries
           if rewrite:
              entry.writeRDF(entryDir)              
           if (entry == None):
              logging.warning(entryDir + " doesn't look as kraken dir")
              return
           entry.parentDirName = directory
           self.entries.append(entry)
           self.__updateKeywords(entry)

           
    def __updateKeywords(self, entry):
        for a in entry.getProperty("keywords"):
           b = string.strip(a)
           if b not in self.keywords:
              self.keywords.append(b) # needed because of some old errors

    def __str__(self):
        x = "Summary of the kraken:"
        x = x + "\n\tThere are " + str(len(self.entries)) + " entries."
        x = x + "\n\tKeywords: " + str(self.keywords)
        return x                        

def generateCache(system, runtime, rewrite = 0):
    TextUi.startLongCommand("Regenerating metadata...")
    temp = MetaData()
    temp.load(system, runtime, rewrite)
    TextUi.endLongCommand("Regenerating metadata done", newline = 1)    
    updateCache(system, runtime, temp)
    return temp

def generateRdf(system, runtime, metadata):    
    TextUi.startLongCommand("Now writing RDF...")
    temp = MetaData()
    temp.load(system, runtime)
    metadata.writeToRdf(os.path.expanduser(kraken.Constants.METADATA_RDF_FILE))    
    TextUi.endLongCommand()    
    
    
# 
#  Used to write out the cache after it was updated with addEntry
#  updateEntry or other stuff like that 
# 
def updateCache(system, runtime, metadata):
    TextUi.startLongCommand("Updating metadata...")
    filename = os.path.expanduser(kraken.Constants.METADATA_FILE)
    FILE = open(filename, "w")
    cPickle.dump(metadata, FILE)
    FILE.close()          
    # and now writing to the meta RDF file    
    TextUi.endLongCommand()

           
def loadMetaData(system, runtime = None):
  filename = os.path.expanduser(kraken.Constants.METADATA_FILE)
  if runtime!=None and runtime.forceRegenerateCache:
      return generateCache(system, runtime)
  try:
    TextUi.startLongCommand("Loading metadata...")
    FILE = open(filename)
    temp = cPickle.load(FILE)
    FILE.close()
    TextUi.endLongCommand()    
    return temp
  except:
    print "Can not open metadata file " + filename
    return generateCache(system, runtime)
    

    
