import os
import os.path
import string
import shutil
import copy
import logging
import Utils2
import kraken.Constants
import TextUi


# **************************************************
#   Helper function: writing out outputs. 
# **************************************************
def writeOutput(system, outputFile, temp):
  if outputFile == "":
     print temp
  else:
     logging.info("Writing to output file:" + outputFile)
     OUTPUTFILE = open(outputFile, "w")
     OUTPUTFILE.write(temp)
     OUTPUTFILE.close()
     logging.info("Done.")

# *********************************************************
#   Function for exit, error message etc. 
# **********************************************************
def cleanExit(system, runtime, exitValue = 0, message = ""):
    if message != "":
        print message        
    os._exit(exitValue)    
   

def cleanStaging(system):   
    # make sure staging is empty
    tmpDir = system.getProperty("tempDir")
    if not os.path.isdir(tmpDir):
        print "Creating staging directory: " + tmpDir
        os.mkdir(tmpDir)
        return
    val = os.listdir(tmpDir)
    if len(val) != 0:
        print "Staging directory " + tmpDir + " is not empty! (probably a failure)"
        if TextUi.confirm("Should I clean it?"):
            for root, dirs, files in os.walk(tmpDir, topdown=False):
               for name in files:
                  os.remove(os.path.join(root, name))
               for name in dirs:
                  os.rmdir(os.path.join(root, name))
            # os.mkdir(tmpDir)
        
# ****************************************************
#  checks a kraken  directory and creates it if necessary
# ****************************************************    
def checkKrakenDir(directory):
       #if directory[-1] != "/":
       #    directory += "/"
       #    # self.dirty = 1       
       val = Utils2.checkAndAttemptToCreateDirectory(directory, "Destination directory")
       if not val:
          return None
       if os.path.exists(directory + kraken.Constants.KRAKEN_DIR_INDICATOR_FILE):
          return directory
       # check if the directory is empty
       FILE = open(directory + kraken.Constants.KRAKEN_DIR_INDICATOR_FILE, "w")
       FILE.write("KRAKEN")
       FILE.close()
       print "Kraken directory:" + directory + " succesfully initialized"
       return directory
        
# *********************************************************
#   Returns the largest entry in the destination directory
# **********************************************************
def createEntryName(system, runtime):
    if not checkKrakenDir(system.getProperty("destinationDir")):
        cleanExit(system, runtime, 1, "Do not have a working destination directory, leaving.")
    currentMax = 0
    for x in os.listdir(system.getProperty("destinationDir")):
        if (os.path.isdir(os.path.join(system.getProperty("destinationDir"), x)) 
           and x[0:len(kraken.Constants.DIRECTORY_PREFIX)] == kraken.Constants.DIRECTORY_PREFIX):
            count = int(x[len(kraken.Constants.DIRECTORY_PREFIX):])
            if count > currentMax: currentMax = count
    return kraken.Constants.DIRECTORY_PREFIX+string.zfill(currentMax+1,5)
  
        
# **********************************************************
#  Create the entry directory
# ***********************************************************
def createEntry(system, runtime):
    runtime.entryDir = createEntryName(system, runtime)
    os.mkdir(os.path.join(system.getProperty("tempDir"), 
            runtime.entryDir))
    os.mkdir(os.path.join(system.getProperty("tempDir"), 
            runtime.entryDir, 
            kraken.Constants.DIRNAME_METADATA))

            
            
            

# **************************************************
#   Move the entry to the destination directory
# **************************************************
def moveEntryToDestinationDir(system, runtime):
    shutil.copytree(os.path.join(system.getProperty("tempDir"), runtime.entryDir),
                    os.path.join(system.getProperty("destinationDir"),runtime.entryDir))
    shutil.rmtree(os.path.join(system.getProperty("tempDir"),runtime.entryDir))

    
def runCommand(commandname, system, runtime):
    '''
    Running a command
    '''
    # first, see if all we are asked is to describe a command
    if runtime.commandToDescribe != "":
       name = "kraken.commands." + runtime.commandToDescribe
       try:
          __import__(name, globals(), locals(), ['execute'])
       except:
          traceback.print_exc()
          cleanExit(system, runtime, 1, "Command: "+ 
              runtime.commandToDescribe + " does not seem to exist.")
       try:   
          t = eval(name +".description(system, runtime)")
          print "-" * 72
          print "\nCommand: " + runtime.commandToDescribe
          print "\nDescription:\n"
          print t
          print "-" * 72 + "\n"   
          return
       except:
          cleanExit(system, runtime, 1, "Command: " + 
              runtime.commandToDescribe + " does not have description.")        
    
    logging.info("Command: " + commandname)
    if commandname == "":
        commandname = "Menu"
    name = "kraken.commands." + commandname
    if 0: # 1 normally, 0 for debug mode
      try:
         __import__(name, globals(), locals(), ['execute'])
      except:
         cleanExit(system, runtime, 1, "Command: "+ commandname + " is not available.")
    else:
      __import__(name, globals(), locals(), ['execute'])
    eval(name +".execute(system, runtime)")

