"""
Python-PDI - Library for Personal Data Interchange.
Copyright (C) 2002-2003 Peter Gebauer

This is the iCalendar base library.

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
version 2 as published by the Free Software Foundation.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
"""

from pdi.core import *
from pdi.vcalendar import *

class ICalendar(VCalendar):
    """This is supposed to be the RFC2445 compliant iCalendar component."""

    def __init__(self, parent = None):
        """
        @type   parent: L{pdi.core.Component}
        @param  parent: The parent component, if any. May be omitted or None if it is a top-level component.
        """
        super(ICalendar, self).__init__(parent)
        self.registerComponents([VJournal], RULE_MAY)
        self.registerComponents([ICalendar], RULE_NOT)
        self.registerProperties(['CALSCALE'], RULE_MUST)
        self.registerProperties(['X-WR-CALNAME',
                                 'X-WR-RELCALID',
                                 'X-WR-TIMEZONE'
                                 ], RULE_MAY)

    def getName(self):
        """
        Because of the crappy iCalendar standard still uses VCALENDAR sections, pretend the name is VCALENDAR.
        @rtype: string
        @return: A constant string 'VCALENDAR'.
        """
        return "VCALENDAR"
